//////////////////////////////////////////////////////////////////////
// Class CListPorts Implementation File

#include "stdafx.h"
#include <string.h>
#include "ListPorts.h"

static BOOL Win9xListPorts(LISTPORTS_CALLBACK lpCbk,LPVOID lpCbkValue);

static BOOL WinNTListPorts(LISTPORTS_CALLBACK lpCbk,LPVOID lpCbkValue);

static BOOL ScanEnumTree(LPCTSTR lpEnumPath,LISTPORTS_CALLBACK lpCbk,LPVOID lpCbkValue);
static LONG OpenSubKeyByIndex(HKEY hKey,DWORD dwIndex,REGSAM samDesired,PHKEY phkResult); 
static LONG QueryStringValue(HKEY hKey,LPCTSTR lpValueName, LPTSTR* lppStringValue);

CListPorts::CListPorts()
{	
}

CListPorts::~CListPorts()
{
}

BOOL CListPorts::ListPorts(LISTPORTS_CALLBACK lpCbk,LPVOID lpCbkValue)
{
	OSVERSIONINFO osvinfo;
	// check parameters 
	if(!lpCbk)
	{
		SetLastError(ERROR_INVALID_PARAMETER);
		return FALSE;
	}

	// determine which platform we're running on and forward
	// to the appropriate routine

	ZeroMemory(&osvinfo,sizeof(osvinfo));
	osvinfo.dwOSVersionInfoSize=sizeof(osvinfo);

	GetVersionEx(&osvinfo);

	switch(osvinfo.dwPlatformId)
	{
	case VER_PLATFORM_WIN32_WINDOWS:
		return Win9xListPorts(lpCbk,lpCbkValue);
		break;
    case VER_PLATFORM_WIN32_NT:
		if(osvinfo.dwMajorVersion<4)
		{
			SetLastError(ERROR_OLD_WIN_VERSION);
			return FALSE;
		}
		return WinNTListPorts(lpCbk,lpCbkValue);
		
		break;
    default:
		SetLastError(ERROR_OLD_WIN_VERSION);
		return FALSE;
		break;
	}
}

//static function
BOOL Win9xListPorts(LISTPORTS_CALLBACK lpCbk,LPVOID lpCbkValue)
{
	return ScanEnumTree(TEXT("ENUM"),lpCbk,lpCbkValue);
}

static BOOL WinNTListPorts(LISTPORTS_CALLBACK lpCbk,LPVOID lpCbkValue)
{
	DWORD  dwError=0;
	HKEY   hKey=NULL;
	DWORD  dwIndex;
	LPTSTR lpValueName=NULL;
	LPTSTR lpPortName=NULL;

	if(dwError=RegOpenKeyEx(HKEY_LOCAL_MACHINE,TEXT("HARDWARE\\DEVICEMAP\\SERIALCOMM"),0,KEY_READ,&hKey))
	{
		if(dwError==ERROR_FILE_NOT_FOUND)
			dwError=0;
		goto end;
	}

	for(dwIndex=0;;++dwIndex)
	{
		DWORD              cbValueName=32*sizeof(TCHAR);
		DWORD              cbPortName=32*sizeof(TCHAR);
		LISTPORTS_PORTINFO portinfo;

		/* loop asking for the value data till we allocated enough memory */

		for(;;)
		{
			free(lpValueName);
			if(!(lpValueName=(LPTSTR)malloc(cbValueName)))
			{
				dwError=ERROR_NOT_ENOUGH_MEMORY;
				goto end;
			}
			free(lpPortName);
			if(!(lpPortName=(LPTSTR)malloc(cbPortName)))
			{
				dwError=ERROR_NOT_ENOUGH_MEMORY;
				goto end;
			}
			if(!(dwError=RegEnumValue(hKey,dwIndex,lpValueName,&cbValueName,
                                NULL,NULL,(LPBYTE)lpPortName,&cbPortName)))
			{
				break; /* we did it */
			}
			else if(dwError==ERROR_MORE_DATA) /* not enough space */
			{ 
				dwError=0;
				/* no indication of space required, we try doubling */
				cbValueName=(cbValueName+sizeof(TCHAR))*2;
			}
			else goto end;
		}

		portinfo.lpPortName=lpPortName;
		portinfo.lpFriendlyName=lpPortName; /* no friendly name in NT 4.0 */
		if(!lpCbk(lpCbkValue,&portinfo))
		{
			goto end; /* listing aborted by callback */
		}
	} 
end:
	free(lpValueName);
	free(lpPortName);
	if(hKey!=NULL)
		RegCloseKey(hKey);
	if(dwError!=0)
	{
		SetLastError(dwError);
		return FALSE;
	}
	else return TRUE;
}

BOOL ScanEnumTree(LPCTSTR lpEnumPath,LISTPORTS_CALLBACK lpCbk,LPVOID lpCbkValue)
{
	static const TCHAR lpstrPortsClass[]=    TEXT("PORTS");
	static const TCHAR lpstrPortsClassGUID[]=TEXT("{4D36E978-E325-11CE-BFC1-08002BE10318}");
	DWORD  dwError=0;
	HKEY   hkEnum=NULL;
	DWORD  dwIndex1;
	HKEY   hkLevel1=NULL;
	DWORD  dwIndex2;
	HKEY   hkLevel2=NULL;
	DWORD  dwIndex3;
	HKEY   hkLevel3=NULL;
	HKEY   hkDeviceParameters=NULL;
	TCHAR  lpClass[sizeof(lpstrPortsClass)/sizeof(lpstrPortsClass[0])];
	DWORD  cbClass;
	TCHAR  lpClassGUID[sizeof(lpstrPortsClassGUID)/sizeof(lpstrPortsClassGUID[0])];
	DWORD  cbClassGUID;
	LPTSTR lpPortName=NULL;
	LPTSTR lpFriendlyName=NULL;
  
	if(dwError=RegOpenKeyEx(HKEY_LOCAL_MACHINE,lpEnumPath,0,KEY_ENUMERATE_SUB_KEYS,&hkEnum))
	{
		goto end;
	}

	for(dwIndex1=0;;++dwIndex1)
	{
		if(hkLevel1!=NULL)
		{
			RegCloseKey(hkLevel1);
			hkLevel1=NULL;
		}
		if(dwError=OpenSubKeyByIndex(hkEnum,dwIndex1,KEY_ENUMERATE_SUB_KEYS,&hkLevel1))
		{
			if(dwError==ERROR_NO_MORE_ITEMS)
			{
				dwError=0;
				break;
			}
			else goto end;
		}

		for(dwIndex2=0;;++dwIndex2)
		{
			if(hkLevel2!=NULL)
			{
				RegCloseKey(hkLevel2);
				hkLevel2=NULL;
			}
			if(dwError=OpenSubKeyByIndex(hkLevel1,dwIndex2,KEY_ENUMERATE_SUB_KEYS,&hkLevel2))
			{
				if(dwError==ERROR_NO_MORE_ITEMS)
				{
					dwError=0;
					break;
				}
				else goto end;
			}

			for(dwIndex3=0;;++dwIndex3)
			{
				BOOL               bFriendlyNameNotFound=FALSE;
				LISTPORTS_PORTINFO portinfo;

				if(hkLevel3!=NULL)
				{
					RegCloseKey(hkLevel3);
					hkLevel3=NULL;
				}
				if(dwError=OpenSubKeyByIndex(hkLevel2,dwIndex3,KEY_READ,&hkLevel3))
				{
					if(dwError==ERROR_NO_MORE_ITEMS)
					{
						dwError=0;
						break;
					}
					else goto end;
				}

				cbClass=sizeof(lpClass);
				if(RegQueryValueEx(hkLevel3,TEXT("CLASS"),NULL,NULL,
		                     (LPBYTE)lpClass,&cbClass)==ERROR_SUCCESS&&
							_tcsicmp(lpClass,lpstrPortsClass)==0)
				{
					/* ok */
				}
				else
				{
					cbClassGUID=sizeof(lpClassGUID);
					if(RegQueryValueEx(hkLevel3,TEXT("CLASSGUID"),NULL,NULL,
						         (LPBYTE)lpClassGUID,&cbClassGUID)==ERROR_SUCCESS&&
								_tcsicmp(lpClassGUID,lpstrPortsClassGUID)==0)
					{
						/* ok */
					}
					else continue;
				}

				/* get "PORTNAME" */

				dwError=QueryStringValue(hkLevel3,TEXT("PORTNAME"),&lpPortName);
				if(dwError==ERROR_FILE_NOT_FOUND)
				{
					if(hkDeviceParameters!=NULL)
					{
						RegCloseKey(hkDeviceParameters);
						hkDeviceParameters=NULL;
					}
					if(RegOpenKeyEx(hkLevel3,TEXT("DEVICE PARAMETERS"),0,KEY_READ,
	                          &hkDeviceParameters)==ERROR_SUCCESS)
					{
						dwError=QueryStringValue(hkDeviceParameters,TEXT("PORTNAME"),&lpPortName);
					}
				}	
				if(dwError)
				{
					if(dwError==ERROR_FILE_NOT_FOUND)
					{ 
						dwError=0;
						continue;
					}
					else goto end;
				}

				/* check if it is a serial port (instead of, say, a parallel port) */
	
				if(_tcsncicmp(lpPortName,TEXT("COM"),3)!=0)
					continue;
	
				/* now go for "FRIENDLYNAME" */
	
				if(dwError=QueryStringValue(hkLevel3,TEXT("FRIENDLYNAME"),&lpFriendlyName))
				{
					if(dwError==ERROR_FILE_NOT_FOUND)
					{
						bFriendlyNameNotFound=TRUE;
						dwError=0;
					}
					else goto end;
				}
        
				/* Assemble the information and pass it on to the callback.
				 * In the unlikely case there's no friendly name available,
				 * use port name instead.
			     */
				portinfo.lpPortName=lpPortName;
				portinfo.lpFriendlyName=bFriendlyNameNotFound?lpPortName:lpFriendlyName;
				if(!lpCbk(lpCbkValue,&portinfo))
				{
					goto end; /* listing aborted by callback */
				}
			}
		}
	}	

end:
	free(lpFriendlyName);
	free(lpPortName);
	if(hkDeviceParameters!=NULL)
		RegCloseKey(hkDeviceParameters);
	if(hkLevel3!=NULL)
		RegCloseKey(hkLevel3);
	if(hkLevel2!=NULL)          
		RegCloseKey(hkLevel2);
	if(hkLevel1!=NULL)          
		RegCloseKey(hkLevel1);
	if(hkEnum!=NULL)            
		RegCloseKey(hkEnum);
	if(dwError!=0)
	{
		SetLastError(dwError);
		return FALSE;
	}
	else return TRUE;
}

LONG OpenSubKeyByIndex(HKEY hKey,DWORD dwIndex,REGSAM samDesired,PHKEY phkResult)
{
  DWORD              dwError=0;
  LPTSTR             lpSubkeyName=NULL;
  DWORD              cbSubkeyName=128*sizeof(TCHAR); /* an initial guess */
  FILETIME           filetime;

  /* loop asking for the subkey name till we allocated enough memory */

  for(;;){
    free(lpSubkeyName);
    if(!(lpSubkeyName=(LPTSTR)malloc(cbSubkeyName))){
       dwError=ERROR_NOT_ENOUGH_MEMORY;
       goto end;
    }
    if(!(dwError=RegEnumKeyEx(hKey,dwIndex,lpSubkeyName,&cbSubkeyName,
                              0,NULL,NULL,&filetime))){
      break; /* we did it */
    }
    else if(dwError==ERROR_MORE_DATA){ /* not enough space */
      dwError=0;
      /* no indication of space required, we try doubling */
      cbSubkeyName=(cbSubkeyName+sizeof(TCHAR))*2;
    }
    else goto end;
  }

  dwError=RegOpenKeyEx(hKey,lpSubkeyName,0,samDesired,phkResult);

end:
  free(lpSubkeyName);
  return dwError;
}

LONG QueryStringValue(HKEY hKey,LPCTSTR lpValueName,LPTSTR* lppStringValue)
{
  DWORD cbStringValue=128*sizeof(TCHAR); /* an initial guess */

  for(;;){
    DWORD dwError;

    free(*lppStringValue);
    if(!(*lppStringValue=(LPTSTR)malloc(cbStringValue))){
      return ERROR_NOT_ENOUGH_MEMORY;
    }
    if(!(dwError=RegQueryValueEx(hKey,lpValueName,NULL,NULL,
                                 (LPBYTE)*lppStringValue,&cbStringValue))){
      return ERROR_SUCCESS;
    }
    else if(dwError==ERROR_MORE_DATA){
      /* not enough space, keep looping */
    }
    else return dwError;
  }
}