// ServiceCtrl.cpp: implementation of the CServiceCtrl class.
//
//////////////////////////////////////////////////////////////////////
#include "StdAfx.h"
#include <stdio.h>
#include <windows.h>
#include <commctrl.h>
#include <stdlib.h>
#include <tchar.h>
#include <shellapi.h>
#include <accctrl.h>
#include <aclapi.h>

#include "ServiceCtrl.h"
#include "EventRes.h"

CServiceCtrl* CServiceCtrl::m_pThis = NULL;


//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////
CServiceCtrl::CServiceCtrl()
{
	m_hInst				= NULL;
	m_hAgentCommWnd		= NULL;
	m_hStopServiceHandle= NULL; // event handle for stopping Agent
	m_pThis				= this;

	m_sshStatusHandle					= NULL;
	m_ssStatus.dwServiceType			= SERVICE_WIN32_OWN_PROCESS;
	m_ssStatus.dwCurrentState			= SERVICE_STOPPED;
	m_ssStatus.dwControlsAccepted		= SERVICE_ACCEPT_STOP;
	m_ssStatus.dwWin32ExitCode			= 0;
	m_ssStatus.dwServiceSpecificExitCode= 0;
	m_ssStatus.dwCheckPoint				= 0;
	m_ssStatus.dwWaitHint				= 0;
}

CServiceCtrl::~CServiceCtrl()
{
}

//	Name:
//		InstallService
//
//	Purpose:
//		Create the service
//
//	Arguments:
//		none	
//
//	Returns:
//		TRUE if Successful, or FALSE
/////////////////////////////////////////////
BOOL CServiceCtrl::InstallService ()
{
		SC_HANDLE	schService;
		SC_HANDLE	schManager;
		
		TCHAR		tszDependencies[] = "";
		TCHAR		tszPath[512] = {0};
		
		//Get the service file
		if ( GetModuleFileName(NULL, tszPath, 512) == 0 ) //fails
			return FALSE;
		
		schManager = OpenSCManager(	
			NULL,					//Local machine(default machine)
			NULL,					//database(default database)
			SC_MANAGER_ALL_ACCESS);	//desired access
		
		if ( schManager == NULL )//SCManager database open fails
		{
			return FALSE;
		}

		if (m_upsCheck.GetOSVersion_Vista() >= 0)
		{
			schService = CreateService(	schManager,					//SCManager database
				SZSERVICENAME,				//name of service
				SZDISPLAYNAME,				//name of display name
				SERVICE_ALL_ACCESS,			//desired access
				SERVICE_WIN32_OWN_PROCESS, //service type
				SERVICE_AUTO_START,			//start type
				SERVICE_ERROR_NORMAL,		//error control type
				tszPath,					//service's binary
				NULL,						//no load ordering group
				NULL,						//no tag identifier
				tszDependencies,			//dependencies
				NULL,						//Localsystem account
				NULL);						//no password
		}
		else
		{
			schService = CreateService(	schManager,					//SCManager database
				SZSERVICENAME,				//name of service
				SZDISPLAYNAME,				//name of display name
				SERVICE_ALL_ACCESS,			//desired access
				SERVICE_WIN32_OWN_PROCESS |	SERVICE_INTERACTIVE_PROCESS, //service type
				SERVICE_AUTO_START,			//start type
				SERVICE_ERROR_NORMAL,		//error control type
				tszPath,					//service's binary
				NULL,						//no load ordering group
				NULL,						//no tag identifier
				tszDependencies,			//dependencies
				NULL,						//Localsystem account
				NULL);						//no password
		}
				
		if ( schService == NULL )//Create fails
		{
			CloseServiceHandle(schManager);
			return FALSE;
		}

		SetServiceDescription(DESCRIPTION_SERVICE);

		AdjustAccessRights(SZSERVICENAME);
		::StartService(schService, 0, NULL);//start service as soon as install OK
		CloseServiceHandle(schService);		//close the service handle
		CloseServiceHandle(schManager);		//close the control manager
		
		return TRUE;
}

BOOL CServiceCtrl::AdjustAccessRights(char *lpszServiceName) 
{
	BOOL					bDaclPresent = FALSE;
	BOOL					bDaclDefaulted = FALSE;
	SC_HANDLE				schManager = NULL;
	SC_HANDLE				schService = NULL;
	PACL					pacl = NULL;
	PACL					pnewacl = NULL;
	EXPLICIT_ACCESS			exa;
	DWORD					dwSize = 0;
	DWORD					code = ERROR_SUCCESS;
	SECURITY_DESCRIPTOR		sd;
	PSECURITY_DESCRIPTOR	psdesc = NULL;
	BOOL					bError = FALSE;
	
	memset(&exa, 0, sizeof(exa));
	memset(&sd, 0, sizeof(sd));
	
	schManager = OpenSCManager(NULL, NULL, SC_MANAGER_CONNECT);
	if(!schManager)
	{
		bError = TRUE;
	}
	if(!bError)
	{
		schService = OpenService(schManager, lpszServiceName, READ_CONTROL | WRITE_DAC);
		if(!schService) 
		{
			bError = TRUE;
		}
	}
	
	if(!bError)
	{
		if (!QueryServiceObjectSecurity(schService, DACL_SECURITY_INFORMATION, &sd, 0, &dwSize))
		{
			if (GetLastError() == ERROR_INSUFFICIENT_BUFFER)
			{
				psdesc = (PSECURITY_DESCRIPTOR)HeapAlloc(GetProcessHeap(), HEAP_ZERO_MEMORY, dwSize);
				if (psdesc == NULL)
				{
					bError = TRUE;
				}
				
				if (!QueryServiceObjectSecurity(schService,	DACL_SECURITY_INFORMATION, psdesc, dwSize, &dwSize)) 
				{
					bError = TRUE;
				}
			}
			else 
			{
				bError = TRUE;
			}
		}
	}

	if(!bError)
	{
		if (!GetSecurityDescriptorDacl(psdesc, &bDaclPresent, &pacl, &bDaclDefaulted))
		{
			bError = TRUE;
		}
		BuildExplicitAccessWithName(&exa, 
									"Everyone",
									SPECIFIC_RIGHTS_ALL | STANDARD_RIGHTS_ALL,
									SET_ACCESS,
									NO_INHERITANCE);
	
		code = SetEntriesInAcl(1, &exa, pacl, &pnewacl);
		if(code != ERROR_SUCCESS) 
		{
			bError = TRUE;
		}

		if(!InitializeSecurityDescriptor(&sd, SECURITY_DESCRIPTOR_REVISION))
		{
			bError = TRUE;
		}
		if(!SetSecurityDescriptorDacl(&sd, TRUE, pnewacl, FALSE))
		{
			bError = TRUE;
		}
				
		if (!SetServiceObjectSecurity(schService, DACL_SECURITY_INFORMATION, &sd))
		{
			bError = TRUE;
		}
	}		

	if(pnewacl)
	{	
		LocalFree(pnewacl);
	}
	if(psdesc)
	{
		HeapFree(GetProcessHeap(), 0, psdesc);
	}
	if(schService)
	{
		CloseServiceHandle(schService);
	}
	if(schManager)
	{
		CloseServiceHandle(schManager);
	}
	
	return bError;
}

//	Name:
//		RemoveService
//
//	Purpose:
//		Uninstall the service from system
//
//	Arguments:
//		none
//
//	Returns:
//		TRUE if Successful, or FALSE
/////////////////////////////////////////////

BOOL CServiceCtrl::RemoveService ()
{
		SC_HANDLE	schService;
		SC_HANDLE	schManager;
		
		SERVICE_STATUS	ssStatus;
		
		schManager = OpenSCManager(	NULL,					//Local machine(default machine)
			NULL,					//database(default database)
			SC_MANAGER_ALL_ACCESS);	//desired access
		
		if ( schManager )//open database OK
		{
			schService = OpenService(	schManager,			//SCManager database
				SZSERVICENAME,		//name of service		
				SERVICE_ALL_ACCESS);//desired access
			
			if ( schService )//open service OK
			{
				//if the service is running, then stop it
				if ( ControlService(schService, SERVICE_CONTROL_STOP, &ssStatus) )
				{
					Sleep(1000);
					
					while ( QueryServiceStatus(schService, &ssStatus) )//get current status
					{
						
						if ( ssStatus.dwCurrentState == SERVICE_STOP_PENDING )//is stopping, then wait
							Sleep(500);
						else//stop successfully
							break;
					}
				}
				DeleteService(schService);								
				CloseServiceHandle(schService);
			}
			else//open service error
			{
				return FALSE;
			}
			
			CloseServiceHandle(schManager);
		}

		else//open database error
		{
			return FALSE;
		}

		
		return TRUE;
}

//	Name:
//		StartService
//
//	Purpose:
//		start the service routine
//
//	Arguments:
//		none
//
//	Returns:
//		TRUE if Successful, or FALSE
/////////////////////////////////////////////
BOOL CServiceCtrl::StartService ()
{
		SERVICE_TABLE_ENTRY dispatchTable[] =
		{
			{ TEXT(SZSERVICENAME), (LPSERVICE_MAIN_FUNCTION)ServiceMain },
			{ NULL, NULL }
		};
		
		//set the service_main function
		if ( !StartServiceCtrlDispatcher(dispatchTable) )
			return FALSE;
		
		return TRUE;
}

//	Name:
//		StopService
//
//	Purpose:
//		stop the service routine
//
//	Arguments:
//		none
//
//	Returns:
//		none
/////////////////////////////////////////////
void CServiceCtrl::StopService ()
{
	//check the status of the stop_service handle
	if ( m_hStopServiceHandle )
		SetEvent(m_hStopServiceHandle);
}

//	Name:
//		ServiceMain
//
//	Purpose:
//		the main function of the service
//
//	Arguments:
//		dwArgc		--	the number of parameter
//		lpszArgv	--	points to the string who contains the parameter
//
//	Returns:
//		none
/////////////////////////////////////////////
void WINAPI CServiceCtrl::ServiceMain (DWORD dwArgc, LPTSTR* lpszArgv)
{
	BOOL ret;
	
	UNREFERENCED_PARAMETER(dwArgc);
	UNREFERENCED_PARAMETER(lpszArgv);
	//Register Control handle function
	m_pThis->m_sshStatusHandle = RegisterServiceCtrlHandler(
		SZSERVICENAME,		//name of service
		(LPHANDLER_FUNCTION)(CServiceCtrl::ServiceHandle)
		);
	
	ret = m_pThis->ModifyServiceStatus(SERVICE_START_PENDING, NO_ERR, 3000);
	
	//Initialization
	if ( !m_pThis->InitialService())
	{
		m_pThis->ModifyServiceStatus(SERVICE_STOPPED, NO_ERR, 0);
		return;
	}
	// run untill te stop event is set
	
	m_pThis->Run();
}

//	Name:
//		ServiceHandle
//
//	Purpose:
//		the control handle function of the service
//
//	Arguments:
//		dwControlID	--	the control ID that system notifies
//
//	Returns:
//		none
/////////////////////////////////////////////
void WINAPI CServiceCtrl::ServiceHandle (DWORD dwControlID)
{
	switch ( dwControlID )
	{
	case SERVICE_CONTROL_STOP:
		m_pThis->ModifyServiceStatus(SERVICE_STOP_PENDING, NO_ERR, 1000);
		m_pThis->StopService();
		return;

	case SERVICE_CONTROL_SHUTDOWN:
		m_pThis->ModifyServiceStatus(SERVICE_STOP_PENDING, NO_ERR, 1000);
		m_pThis->StopService();
		return;

	default:
		m_pThis->ModifyServiceStatus(m_pThis->m_ssStatus.dwCurrentState, NO_ERR, 0);
		return;
	}
}


//	Name:
//		InitialService
//
//	Purpose:
//		initialize the service
//
//	Arguments:
//		none
//
//	Returns:
//		TRUE if Successful, or FALSE
/////////////////////////////////////////////
BOOL CServiceCtrl::InitialService ()
{
	BOOL bRet = TRUE;
	m_hStopServiceHandle = CreateEvent(NULL, TRUE, FALSE, 
		"StopAgent");	//name of service
	
	if ( !m_hStopServiceHandle )
		return FALSE;
	
	if ( GetLastError() == ERROR_ALREADY_EXISTS )
		return FALSE;

	bRet = m_upsCheck.Initialize();

	Sleep(1000);

	return bRet;
}

//	Name:
//		ModifyServiceStatus
//
//	Purpose:
//		modify the current status of service
//
//	Arguments:
//		dwCurrentState	--	the current status
//		dwWin32ExitCode	--	the code of exit
//		dwWaitHint		--	the time for waiting
//
//	Returns:
//		TRUE if Successful, or FALSE
/////////////////////////////////////////////
BOOL CServiceCtrl::ModifyServiceStatus (DWORD	dwCurrentState,
										 DWORD	dwWin32ExitCode,
										 DWORD	dwWaitHint)
{
	static DWORD dwCheckPoint = 1;
	
	if (dwCurrentState == SERVICE_START_PENDING)
		m_ssStatus.dwControlsAccepted =  0;
	else
		m_ssStatus.dwControlsAccepted = SERVICE_ACCEPT_STOP|SERVICE_ACCEPT_SHUTDOWN;
	
	m_ssStatus.dwCurrentState	= dwCurrentState;
	m_ssStatus.dwWin32ExitCode	= dwWin32ExitCode;
	m_ssStatus.dwWaitHint		= dwWaitHint;
	
	if ( ( dwCurrentState == SERVICE_RUNNING ) ||
		( dwCurrentState == SERVICE_STOPPED ) )
		m_ssStatus.dwCheckPoint = 0;
	else
		m_ssStatus.dwCheckPoint = dwCheckPoint++;
	
	return SetServiceStatus(m_sshStatusHandle, &m_ssStatus);
}

BOOL CServiceCtrl::SetServiceDescription(LPCSTR lpDescription)
{
	BOOL bRet = FALSE;

	SC_LOCK sclLock; 
	SC_HANDLE	hSCManager, hService;

	hSCManager = OpenSCManager (NULL,NULL,SC_MANAGER_ALL_ACCESS);
	if (hSCManager )
	{
		// Need to acquire database lock before reconfiguring. 
		sclLock = LockServiceDatabase(hSCManager); 

		if (sclLock == NULL)
		{
			CloseServiceHandle(hSCManager);
			return FALSE;
		}
		// The database is locked, so it is safe to make changes. 
		// Open a handle to the service
		hService = OpenService (hSCManager,SZSERVICENAME,SERVICE_CHANGE_CONFIG);
		if(hService)
		{
			SERVICE_DESCRIPTION  ServDesp;
			ServDesp.lpDescription = (LPTSTR)lpDescription;
			if(ChangeServiceConfig2(hService,
				SERVICE_CONFIG_DESCRIPTION,
				&ServDesp))
			{
				bRet = TRUE;
			}
			DWORD dwErr = GetLastError();
			CloseServiceHandle (hService);									
		}
		UnlockServiceDatabase(sclLock); 
		CloseServiceHandle(hSCManager);
	}

	return bRet;

}
//	Name:
//		Run
//
//	Purpose:
//		run till the stop_service event is set
//
//	Arguments:
//		none
//
//	Returns:
//		none
/////////////////////////////////////////////
void CServiceCtrl::Run()
{	
	UINT uShutdownStatus = SHUTDOWN_NONE;
	BOOL bUPSShutdown = FALSE;
	BOOL bShutdown = FALSE;
	HANDLE	hUPSCMDSNEvent = OpenEvent ( EVENT_ALL_ACCESS, FALSE, UPS_CMD_SN);

	ModifyServiceStatus(SERVICE_RUNNING, NO_ERR, 0);
	
	m_upsCheck.SimpleSDLogEvent(EVENTLOG_INFORMATION_TYPE, UPS_EVENT_SERVICE_START, ID_AGENT_START);

	while ( WaitForSingleObject(m_hStopServiceHandle, 0) != WAIT_OBJECT_0 )
	{
		m_upsCheck.UpdateConfigFile();
		
		if(m_upsCheck.CheckEvent())
		{
			if (!bUPSShutdown)
			{
				if( !m_upsCheck.DoDelay())
				{
					//imachine added 2010.07.12
					//if( WaitForSingleObject( hUPSCMDSNEvent, 100 ) == WAIT_OBJECT_0 )
					{
						bUPSShutdown = TRUE;
						//ResetEvent( hUPSCMDSNEvent );
					}
				}
			}
		}
		
		if (bUPSShutdown)
		{
			if(m_upsCheck.DoShutdownLogic())
			{
				bShutdown =TRUE;
				break;
			}
		}
		Sleep(100);
	}

	if ( m_hAgentCommWnd )
	{
		SendMessage(m_hAgentCommWnd, WM_COMMEND, 0, 0);
		m_hAgentCommWnd = NULL;
	}

	m_upsCheck.SimpleSDLogEvent(EVENTLOG_INFORMATION_TYPE, UPS_EVENT_SERVICE_STOP, ID_AGENT_STOP);

	if(m_upsCheck.m_SDParam.uCommandDelayTime == 0)
	{
		Sleep(500);
	}

	Sleep( 2000 );

	if(bShutdown)
	{
		m_upsCheck.SimpleSDLogEvent(EVENTLOG_INFORMATION_TYPE, UPS_EVENT_SHUTDOWN, ID_SHUTDOWN);
		
		HANDLE hToken;
		TOKEN_PRIVILEGES tkp;  // Get a token for this process.  
		if (!OpenProcessToken(GetCurrentProcess(), 
			TOKEN_ADJUST_PRIVILEGES | TOKEN_QUERY, &hToken)) 
			return;
		LookupPrivilegeValue(NULL, SE_SHUTDOWN_NAME,
			&tkp.Privileges[0].Luid);  
		tkp.PrivilegeCount = 1;  // one privilege to set    
		tkp.Privileges[0].Attributes = SE_PRIVILEGE_ENABLED;  
		
		AdjustTokenPrivileges(hToken, FALSE, &tkp, 0, (PTOKEN_PRIVILEGES)NULL, 0);  

		// The ExitWindowsEx will not turn off OS if screen saver is running,
		// so firstly stop screen saver by calling SetCursorPos before calling ExitWindowsEx.
		SetCursorPos(0, 0);

		if (UPS_STATUS_ONLY_SD_OS == m_upsCheck.m_upsData->m_upsObject.m_nOmronStatus)
		{
			ExitWindowsEx(EWX_REBOOT | EWX_FORCE, 0);
		}
		else
			ExitWindowsEx(EWX_SHUTDOWN | EWX_FORCE, 0);

	}

	if (m_hStopServiceHandle != NULL)
	{
		CloseHandle(m_hStopServiceHandle);
		m_hStopServiceHandle = NULL;
	}

	ModifyServiceStatus(SERVICE_STOPPED, NO_ERR, 0);

}


