// UPSDLL.cpp: implemeint goble_variable_UPS_statusntation of the CUPSDLL class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "UPSDLL.h"
#include "ShareMemUPSCmd.h"
#include "UpsCheck.h"  //Colin@2010-12-14  added for write Eventlog
#include "EventRes.h"

extern int goble_variable_UPS_status;

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif


extern POpenOneUSBDevice OpenOneUSBDevice;
CUPSDLL::CUPSDLL()
{
	m_bRunScan				= TRUE;
	m_hScanUpsThread		= NULL;
	m_hStopAgent			= NULL;
	m_hScanThread			= NULL;
	m_hPipEvent				= NULL;
	m_upsObject.m_nOmronStatus	= 0;
	m_upsObject.m_nRestartTime	= 0;
	m_bSMCmdManuallyInited = m_ShareMem.InitializeMem(1, sizeof(SHM_DATA), UPSDATAMAPPINGNAME, SYNCFILEMAPPING, FALSE);
	m_hUpdateConfigForComm = NULL;

	//imachine added 2010.07.12
	m_hUpdateConnection = NULL;	
	m_hUPSCMDSNEvent = NULL;

	//imachine added 2010.08.24
	m_hUpdateOmronCtrl = NULL;	
}

CUPSDLL::~CUPSDLL()
{
	RemoveUPSObject();
	if (m_hUpdateConfigForComm)
	{
		CloseHandle(m_hUpdateConfigForComm);
	}

	//imachine added 2010.07.12
	if (m_hUpdateConnection)
	{
		CloseHandle( m_hUpdateConnection );
	}	
	if (m_hUPSCMDSNEvent)
	{
		CloseHandle( m_hUPSCMDSNEvent );
	}
	//Added End

	//imachine added 2010.08.24
	if (m_hUpdateOmronCtrl)
	{
		CloseHandle( m_hUpdateOmronCtrl );
	}
}

void CUPSDLL::Terminate()
{
	UINT	uTimeCount = 100;
	UINT	i = 0;
	DWORD	dwStopCount = 0;
	m_bRunScan = FALSE;	


	TerminateThread(m_hScanThread, 0);

	TerminateThread(m_hScanUpsThread, 0);

	if(m_hScanThread)
	{
		CloseHandle(m_hScanThread);
		m_hScanThread = NULL;
	}

	if(m_hScanUpsThread)
	{
		CloseHandle(m_hScanUpsThread);
		m_hScanUpsThread = NULL;
	}

	if (m_hPipEvent)
	{
		CloseHandle(m_hPipEvent);
		m_hPipEvent = NULL;
	}

	RemoveUPSObject();
}

BOOL CUPSDLL::RemoveUPSObject()
{
	if(m_upsObject.m_PortType == COM)
	{
		m_upsObject.CloseConnection(COM);
	}
	else
	{
		m_upsObject.CloseConnection(USB);
	}

	return TRUE;
}

BOOL CUPSDLL::Initialize()
{
	m_bConnect = FALSE;

	SECURITY_ATTRIBUTES sa;
	SECURITY_DESCRIPTOR sd;

	sa.nLength = sizeof(SECURITY_ATTRIBUTES);
	sa.bInheritHandle = TRUE;
	sa.lpSecurityDescriptor = &sd;

	if(!InitializeSecurityDescriptor(&sd,
				SECURITY_DESCRIPTOR_REVISION))
	{
		return false;   // Handle errors.
	}

	// Add a NULL DACL to the security descriptor.
	if(!SetSecurityDescriptorDacl(&sd, TRUE, (PACL)NULL, FALSE))
	{
		return false;   // Handle errors.
	}

	m_hUpdateConfigForComm = CreateEvent(&sa, TRUE, FALSE, UPDATE_CONFIG_FILE_FOR_COMM);
	if ( m_hUpdateConfigForComm == NULL )
	{
		return FALSE;
	}

	//imachine added 2010.07.12
	m_hUpdateConnection = CreateEvent(&sa, TRUE, FALSE, UPDATE_CONNECTION_STATUS);
	if ( m_hUpdateConnection == NULL )
	{
		return FALSE;
	}
	m_hUPSCMDSNEvent = CreateEvent( &sa, TRUE, FALSE, UPS_CMD_SN );
	if ( m_hUPSCMDSNEvent == NULL )
	{
		return FALSE;
	}
	//Added End

	//imachine added 2010.08.24
	m_hUpdateOmronCtrl = CreateEvent(&sa, TRUE, FALSE, UPDATE_OMRON_OPTION);
	if ( m_hUpdateOmronCtrl == NULL )
	{
		return FALSE;
	}

	m_fileConfig.GetShutdownParameter(&m_SDParam);

	m_hPipEvent = CreateEvent(NULL, TRUE, FALSE,READPIP_EVENT_NAME);	//name of service

	if ( !m_hPipEvent )
		return FALSE;

	if(!WaitPipeServ())
		return FALSE;

	if( m_hScanUpsThread =	
			CreateThread((LPSECURITY_ATTRIBUTES)NULL,
				0,
				(LPTHREAD_START_ROUTINE)ScanUpsThread,
				(LPVOID)this,
				0,
				NULL))
	{
		m_bRunScan = TRUE;
		return TRUE;
	}
	else
	{
		m_bRunScan = FALSE;
		return FALSE;
	}
}

void CUPSDLL::UpdateConfigFile()
{	
	if (WaitForSingleObject(m_hUpdateConfigForComm, 100) == WAIT_OBJECT_0)
	{
		memcpy(&m_oldSDParam,&m_SDParam,sizeof(m_SDParam));
		m_fileConfig.GetShutdownParameter(&m_SDParam);
		ResetEvent(m_hUpdateConfigForComm);
	}
}

int  CUPSDLL::DealWithSHMForOmron()
{	
	BOOL ret = FALSE;
	SHM_DATA_OMRON shmdata;
	size_t		i = 0;
	size_t		nSize = MAX_PATH;
	char	szFileName[MAX_PATH] = {0};
	FILE * fp = NULL;
	char szTemp[512];
	CUpsCheck m_upsCheck;

	if (m_cSHMForOmron.Lock(700)) 
	{	
		memset(&shmdata, 0x00, sizeof(SHM_DATA_OMRON));
		m_cSHMForOmron.GetData(0, &shmdata, sizeof(SHM_DATA_OMRON));
		// Colin@2010-12-14  added for print error log
		if (0 != shmdata.uPara3) {
			if (503 == shmdata.uPara3) {
				memset(&shmdata, 0x00, sizeof(SHM_DATA_OMRON));
				if (goble_variable_UPS_status == UPSSTATUS_COMLST){
						shmdata.uPara4 = 502;
					}
				m_cSHMForOmron.SetData(0, &shmdata, sizeof(SHM_DATA_OMRON));
				m_upsCheck.SimpleSDLogEvent(EVENTLOG_INFORMATION_TYPE, UPS_EVENT_OC_ERROR_CMD_EXEC, ID_OC_ERROR_CMD_EXEC);	
				m_cSHMForOmron.Unlock();
				return ret;
			}
			else if (504 == shmdata.uPara3) {
				memset(&shmdata, 0x00, sizeof(SHM_DATA_OMRON));
				if (goble_variable_UPS_status == UPSSTATUS_COMLST){
						shmdata.uPara4 = 502;
					}
				m_cSHMForOmron.SetData(0, &shmdata, sizeof(SHM_DATA_OMRON));
				m_upsCheck.SimpleSDLogEvent(EVENTLOG_INFORMATION_TYPE, UPS_EVENT_OC_ERROR_SET_TIME, ID_OC_ERROR_SET_TIME);
				m_cSHMForOmron.Unlock();
				return ret;
			}
			else if (505 == shmdata.uPara3) {
				memset(&shmdata, 0x00, sizeof(SHM_DATA_OMRON));
				if (goble_variable_UPS_status == UPSSTATUS_COMLST){
						shmdata.uPara4 = 502;
					}
				m_cSHMForOmron.SetData(0, &shmdata, sizeof(SHM_DATA_OMRON));
				m_upsCheck.SimpleSDLogEvent(EVENTLOG_INFORMATION_TYPE, UPS_EVENT_OC_WARRING_RESTART_DELAY, ID_OC_WARRING_RESTART_DELAY);
				m_cSHMForOmron.Unlock();
				return ret;
			}
			else if (506 == shmdata.uPara3) {
				memset(&shmdata, 0x00, sizeof(SHM_DATA_OMRON));
				if (goble_variable_UPS_status == UPSSTATUS_COMLST){
						shmdata.uPara4 = 502;
					}
				m_cSHMForOmron.SetData(0, &shmdata, sizeof(SHM_DATA_OMRON));
				m_upsCheck.SimpleSDLogEvent(EVENTLOG_INFORMATION_TYPE, UPS_EVENT_OC_ERROR_OTHER, ID_OC_ERROR_OTHER);	
				m_cSHMForOmron.Unlock();
				return ret;
			}
		}
		// end Colin add 
			
		if (0 == shmdata.uCmd) 
		{
			ret = FALSE;
		} 
		else 
		{
		//Colin add for write Log about ups communication lost
			if (goble_variable_UPS_status == UPSSTATUS_COMLST) {
				memset(&shmdata, 0x00, sizeof(SHM_DATA_OMRON));
				shmdata.uPara4 = 502;
				m_cSHMForOmron.SetData(0, &shmdata, sizeof(SHM_DATA_OMRON));
		
				m_upsCheck.SimpleSDLogEvent(EVENTLOG_INFORMATION_TYPE, UPS_EVENT_OC_ERROR_COMM_UPS, ID_OC_ERROR_COMM_UPS);
				m_cSHMForOmron.Unlock();

				return ret;
			}
		// end colin add
			
			ret = shmdata.uCmd;
			if (m_upsObject.m_nOmronStatus == 0)
			{
				m_upsObject.m_nOmronStatus = shmdata.uCmd;
				m_upsObject.m_nRestartTime = shmdata.uPara2;
				if (UPS_STATUS_SD_OS_RESTART_LATER == m_upsObject.m_nOmronStatus)
				{
					memset(szFileName, 0x00, nSize);
					if(GetModuleFileName(NULL, szFileName, (DWORD)nSize))
					{
						for(i=strlen(szFileName); i>=0; i--)
						{
							if(szFileName[i]== '\\')
								break;
						}

						szFileName[i+1] = 0;

						strcat(szFileName, "omronctl.ini");
					}
					_unlink(szFileName);
					fp = fopen(szFileName, "w");
					if (NULL != fp)
					{
						memset(szTemp, 0x00, sizeof(szTemp));
						sprintf(szTemp, "%d", m_upsObject.m_nRestartTime);
						fwrite(szTemp, strlen(szTemp), 1, fp);
						fclose(fp);
						fp = NULL;
					}

				}

				//imachine added 2010.08.24
				SetEvent( m_hUpdateOmronCtrl );
			}
			memset(&shmdata, 0x00, sizeof(SHM_DATA_OMRON));
			if (!m_cSHMForOmron.SetData(0, &shmdata, sizeof(SHM_DATA_OMRON))) 
			{
				ret = FALSE;
			} 
		}
		if (goble_variable_UPS_status == UPSSTATUS_COMLST)	
			{
				memset(&shmdata, 0x00, sizeof(SHM_DATA_OMRON));
				m_cSHMForOmron.GetData(0, &shmdata, sizeof(SHM_DATA_OMRON));
				if (shmdata.uPara4 == 0) {
						shmdata.uPara4 = 502;
						m_cSHMForOmron.SetData(0, &shmdata, sizeof(SHM_DATA_OMRON));
					}
			}
		else
			{
				memset(&shmdata, 0x00, sizeof(SHM_DATA_OMRON));
				m_cSHMForOmron.GetData(0, &shmdata, sizeof(SHM_DATA_OMRON));
				if (shmdata.uPara4 == 502) {
						memset(&shmdata, 0x00, sizeof(SHM_DATA_OMRON));
						m_cSHMForOmron.SetData(0, &shmdata, sizeof(SHM_DATA_OMRON));
					}
			}
		m_cSHMForOmron.Unlock();
	}
	
	return ret;

}
DWORD WINAPI ScanUpsThread(LPVOID lpParam)
{
	int comId = 0;
	SHM_DATA_OMRON shmdata;
	CShareMem Judgecomstatus;

	CUPSDLL *pUps = (CUPSDLL*) lpParam;
	pUps->m_hStopAgent = OpenEvent(EVENT_ALL_ACCESS, FALSE, "StopAgent");

	pUps->m_cSHMForOmron.InitializeMem(1, sizeof(SHM_DATA_OMRON), OMRONDATAMAPPINGNAME, OMRONSYNCFILEMAPPING, TRUE);

	if (!pUps->m_upsObject.Initialize())
	{
		if (NULL != pUps->m_hStopAgent)
		{
			CloseHandle(pUps->m_hStopAgent);
			pUps->m_hStopAgent = NULL;
		}
		return 0;
	}

	while(WaitForSingleObject(pUps->m_hStopAgent, 500) != WAIT_OBJECT_0)
	{
		pUps->DealWithSHMForOmron();

		pUps->UpdateConfigFile();

		if(pUps->ReadNamedPipe())
		{
			pUps->m_upsObject.UPSCmdSN(pUps->m_szPipeCmd.nValue);

			//imachine added 2010.07.12
			SetEvent ( pUps->m_hUPSCMDSNEvent );
		}

#if 0
		//colin added for judge comlst
		if (Judgecomstatus.Lock(500)) 
		{
			memset(&shmdata, 0x00, sizeof(SHM_DATA_OMRON));
			Judgecomstatus.GetData(0, &shmdata, sizeof(SHM_DATA_OMRON));
					
			if (goble_variable_UPS_status == UPSSTATUS_COMLST) 
			{
				shmdata.uPara4 = 502;
				Judgecomstatus.SetData(0, &shmdata, sizeof(SHM_DATA_OMRON));
			}
			else 
			{
				shmdata.uPara4 = 0;
				Judgecomstatus.SetData(0, &shmdata, sizeof(SHM_DATA_OMRON));
		
			}
		}
		Judgecomstatus.Unlock();
		//end colin added
#endif
		if (pUps->m_oldSDParam.uSearchType != pUps->m_SDParam.uSearchType)
		{
			if(pUps->m_bConnect)
			{
				pUps->m_bRunScan = FALSE;
				pUps->m_bConnect = FALSE;

				if(pUps->m_hScanThread != NULL)
				{
					TerminateThread(pUps->m_hScanThread, 0);
					CloseHandle(pUps->m_hScanThread);
					pUps->m_hScanThread = NULL;
				}
				if(pUps->m_ShareMem.Lock(500))
				{
					pUps->m_ShareMem.Clear();
					pUps->m_ShareMem.Unlock();
				}
			}

			pUps->m_oldSDParam.uSearchType = pUps->m_SDParam.uSearchType;
		}
		else
		{
			if ((pUps->m_SDParam.uPortType == COM)
					&& (pUps->m_SDParam.uComPort != pUps->m_oldSDParam.uComPort))
			{
				if (pUps->m_bConnect)
				{
					pUps->m_bRunScan = FALSE;
					pUps->m_bConnect = FALSE;

					if(pUps->m_hScanThread != NULL)
					{
						TerminateThread(pUps->m_hScanThread, 0);
						CloseHandle(pUps->m_hScanThread);
						pUps->m_hScanThread = NULL;
					}
					if(pUps->m_ShareMem.Lock(500))
					{
						pUps->m_ShareMem.Clear();
						pUps->m_ShareMem.Unlock();
					}

					CString str1,str2;
					str1.Format("PORTType is:%d",pUps->m_SDParam.uPortType);
				}
			}

		}
		
#if 0
//colin added for judge comlst
		if (Judgecomstatus.Lock(500)) 
		{
			memset(&shmdata, 0x00, sizeof(SHM_DATA_OMRON));
			Judgecomstatus.GetData(0, &shmdata, sizeof(SHM_DATA_OMRON));
					
			if (goble_variable_UPS_status == UPSSTATUS_COMLST) 
			{
				shmdata.uPara4 = 502;
				Judgecomstatus.SetData(0, &shmdata, sizeof(SHM_DATA_OMRON));
			}
			else 
			{
				shmdata.uPara4 = 0;
				Judgecomstatus.SetData(0, &shmdata, sizeof(SHM_DATA_OMRON));
		
			}
		}
		Judgecomstatus.Unlock();
//end colin added
#endif

		if (!pUps->m_bConnect)
		{
			if (pUps->m_SDParam.uSearchType == SEARCHTYPE_DEFAULT)		
			{
				if(!(OpenOneUSBDevice(&(pUps->m_UsbDevice))))			
				{		
					if(pUps->m_upsObject.OpenOneComDevice( &comId,TRUE))
					{
						pUps->m_bConnect = TRUE;
						pUps->m_upsObject.m_upsInfo.dwComID = comId;
						pUps->m_upsObject.m_upsInfo.ptPortType = COM;
					}
				}
				else
				{
					pUps->m_bConnect = TRUE;
					memcpy(&(pUps->m_upsObject.m_UsbDevice),&(pUps->m_UsbDevice),sizeof(USBDNAME));
					pUps->m_upsObject.m_upsInfo.dwComID = pUps->m_UsbDevice.index;
					pUps->m_upsObject.m_upsInfo.ptPortType = USB;
				}
			}
			//mary add 2012.08.29
			else if(pUps->m_SDParam.uSearchType == SEARCHTYPE_SEMI_AUTOMATIC)
			{
				if(OpenOneUSBDevice(&(pUps->m_UsbDevice)))	
				{
					pUps->m_bConnect = TRUE;
					memcpy(&(pUps->m_upsObject.m_UsbDevice),&(pUps->m_UsbDevice),sizeof(USBDNAME));
					pUps->m_upsObject.m_upsInfo.dwComID = pUps->m_UsbDevice.index;
					pUps->m_upsObject.m_upsInfo.ptPortType = USB;
				}
			}
			//mary add end
			else
			{
				comId = pUps->m_SDParam.uComPort;
				if(pUps->m_upsObject.OpenOneComDevice( &comId,FALSE))
				{
					pUps->m_bConnect = TRUE;
					pUps->m_upsObject.m_upsInfo.dwComID = comId;
					pUps->m_upsObject.m_upsInfo.ptPortType = COM;
				}
			}

			if (pUps->m_bConnect)
			{
				pUps->m_oldSDParam.uComPort = pUps->m_SDParam.uComPort = pUps->m_upsObject.m_upsInfo.dwComID;
				pUps->m_oldSDParam.uPortType = pUps->m_upsObject.m_upsInfo.ptPortType = pUps->m_upsObject.m_upsInfo.ptPortType;

				if (NULL != pUps->m_hScanThread)
				{
					CloseHandle(pUps->m_hScanThread);
					pUps->m_hScanThread = NULL;
				}

				if( (pUps->m_hScanThread =	
							CreateThread(
								(LPSECURITY_ATTRIBUTES)NULL,
								0,
								(LPTHREAD_START_ROUTINE)CommitWithUPSThread,
								(LPVOID)lpParam,
								0,
								NULL)) != NULL)
				{
					pUps->m_bRunScan = TRUE;
				}
				else
				{
					pUps->m_bRunScan = FALSE;
					pUps->RemoveUPSObject();
					if (NULL != pUps->m_hStopAgent)
					{
						CloseHandle(pUps->m_hStopAgent);
						pUps->m_hStopAgent = NULL;
					}
					return 0;
				}
			}
		}

#if 0
//colin added for judge comlst
		if (Judgecomstatus.Lock(500)) 
		{
			memset(&shmdata, 0x00, sizeof(SHM_DATA_OMRON));
			Judgecomstatus.GetData(0, &shmdata, sizeof(SHM_DATA_OMRON));
			
			if (goble_variable_UPS_status == UPSSTATUS_COMLST) 
			{
				shmdata.uPara4 = 502;
				Judgecomstatus.SetData(0, &shmdata, sizeof(SHM_DATA_OMRON));
			}
			else 
			{
				shmdata.uPara4 = 0;
				Judgecomstatus.SetData(0, &shmdata, sizeof(SHM_DATA_OMRON));

			}
		}
		Judgecomstatus.Unlock();
//end colin added
#endif
	}
	if (NULL != pUps->m_hStopAgent)
	{
		CloseHandle(pUps->m_hStopAgent);
		pUps->m_hStopAgent = NULL;
	}

	return 1;
}


BOOL CUPSDLL::WaitPipeServ()
{ 
	int i;
	for(i=0; i<3; i++)
	{
		if(WaitNamedPipe(_T(SZ_NAMED_PIPE),100))
			break;
	}
	if (i == 3)
	{
		return FALSE;
	}

	m_hNamedPipe = CreateFile(_T(SZ_NAMED_PIPE), GENERIC_READ|GENERIC_WRITE, 0, NULL, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL);
	if(INVALID_HANDLE_VALUE == m_hNamedPipe)
	{
		return FALSE;
	}

	return TRUE;
}
BOOL CUPSDLL::ReadNamedPipe()
{
	DWORD	dwReadNum;
	DWORD   length;  
	int nRet;
	int cxPipeCmd = sizeof(PIPE_DATA);
	OVERLAPPED overlapped;
	overlapped.Offset = overlapped.OffsetHigh=0;
	overlapped.hEvent = CreateEvent(NULL,TRUE,FALSE,NULL);

	if( WaitForSingleObject(m_hPipEvent,100) == WAIT_OBJECT_0)
	{
		nRet = ReadFile(m_hNamedPipe, &m_szPipeCmd, cxPipeCmd, &dwReadNum, &overlapped);

		if(!nRet && ::GetLastError()   ==   ERROR_IO_PENDING)  
		{
			nRet = GetOverlappedResult(m_hNamedPipe,   &overlapped,   &length,   TRUE);  
		}  


		if( ! nRet)
		{
			if (NULL != overlapped.hEvent)
			{
				CloseHandle(overlapped.hEvent);
				overlapped.hEvent = NULL;
			}

			ResetEvent(m_hPipEvent);
			return FALSE;
		}


		if (NULL != overlapped.hEvent)
		{
			CloseHandle(overlapped.hEvent);
			overlapped.hEvent = NULL;
		}

		ResetEvent(m_hPipEvent);
		return TRUE;
	}
	if (NULL != overlapped.hEvent)
	{
		CloseHandle(overlapped.hEvent);
		overlapped.hEvent = NULL;
	}
	return FALSE;
}

DWORD WINAPI CommitWithUPSThread(LPVOID lpParam)
{
	BOOL	bConnect = FALSE;
	CUPSDLL *pUps = (CUPSDLL*) lpParam;
	DWORD	dwCurrentThreadID = 0;
	UINT	uThreadIndex = UPS_MAX;
	DWORD	dwPortIndex = 0;
	UINT	i = 0;
	CUpsCheck m_upsCheck;

	while(pUps->m_bRunScan)
	{
		if(!bConnect)
		{
			if (pUps->m_upsObject.m_upsInfo.ptPortType == USB)
			{
				dwPortIndex = pUps->m_upsObject.m_upsInfo.dwComID - 1;
			}
			else
			{
				dwPortIndex = pUps->m_upsObject.m_upsInfo.dwComID;
			}
			bConnect = pUps->m_upsObject.ConnectUPS(dwPortIndex, pUps->m_upsObject.m_upsInfo.ptPortType);
			if (!bConnect)
			{
				pUps->m_bConnect = FALSE; 
				break;
			}
		}
		else
		{
			bConnect = pUps->m_upsObject.PollingUPS();
			if(!bConnect)
			{
				memset(&(pUps->m_SHMData),0,sizeof(pUps->m_SHMData));

				if(pUps->m_upsObject.m_hPortHandle)
				{
					CloseHandle(pUps->m_upsObject.m_hPortHandle);
					pUps->m_upsObject.m_hPortHandle = NULL;
				}
				pUps->m_bConnect = FALSE;
				if(pUps->m_ShareMem.Lock(500))
				{
					pUps->m_ShareMem.Clear();
					pUps->m_ShareMem.Unlock();
				}
				break;
			}

			pUps->m_SHMData.uDataAvailable = SHM_DATA_AVAILABLE;
			pUps->m_SHMData.uPortType = pUps->m_upsObject.m_upsInfo.ptPortType;
			pUps->m_SHMData.uPortNum = pUps->m_upsObject.m_upsInfo.dwComID;

			pUps->m_SHMData.dwUPSStatus = pUps->m_upsObject.m_upsInfo.dwUPSStatus;  

			if(pUps->m_ShareMem.Lock(500))
			{
				pUps->m_ShareMem.SetData(0, &(pUps->m_SHMData), SHM_DATA_LENGTH);
				pUps->m_ShareMem.Unlock();
			}
		}

		Sleep(THREAD_DELAY);
	}
	pUps->m_upsObject.CloseConnection(pUps->m_upsObject.m_upsInfo.ptPortType);
	if (NULL != pUps->m_hScanThread)
	{
		CloseHandle(pUps->m_hScanThread);
		pUps->m_hScanThread = NULL;
	}

	return 1;
}



