#include "StdAfx.h"
#include "UpsCheck.h"
#include "ShareMemUPSCmd.h"
#include "..\Protocol.h"
#include <Aclapi.h>
#include "EventRes.h"
#include <Tlhelp32.h>
#include "ServiceCtrl.h"
#include <WtsApi32.h>
#include <Userenv.h>
#include <Winbase.h>

int goble_variable_UPS_status = 0; // Colin@2010-12-16 added for get ups status support for write EventLog

int LOGONEMSGEX(char *pszBuff);


int CUpsCheck::GetOSVersion_Vista()
{
	OSVERSIONINFOEX osvi;
	BOOL bOsVersionInfoEx;

	ZeroMemory(&osvi, sizeof(OSVERSIONINFOEX));

	// Try calling GetVersionEx using the OSVERSIONINFOEX structure.
	// If that fails, try using the OSVERSIONINFO structure.

	osvi.dwOSVersionInfoSize = sizeof(OSVERSIONINFOEX);

	if( !(bOsVersionInfoEx = GetVersionEx ((OSVERSIONINFO *) &osvi)) )
	{
		osvi.dwOSVersionInfoSize = sizeof (OSVERSIONINFO);
		if (! GetVersionEx ( (OSVERSIONINFO *) &osvi) ) 
		{
			return -1;
		}
	}

	switch (osvi.dwPlatformId)
	{
	case VER_PLATFORM_WIN32_NT:
		// Test for the specific product.
		if ( (osvi.dwMajorVersion >= 6) && (osvi.dwMinorVersion >= 0) )
		{
			//vista 
			if( osvi.wProductType == VER_NT_WORKSTATION )
			{
				return 1;
			}
			else	//Windows Server 2008
			{
				return 0;
			}
		}
		break;
	}

	return -1;	
}

CUpsCheck::CUpsCheck(void)
{
	m_bACFail = FALSE;
	m_bBatteryLow = FALSE;
	m_hFileMap = NULL;
	m_hMutex = NULL;
	m_pEveryoneSID = NULL; 
	m_pAdminSID = NULL;
	m_pACL = NULL;
	m_pSD = NULL;
	m_hUpdateConfig = NULL;
	m_hUpdatePip = NULL;
	m_bFirstCommEst = TRUE;
	m_bFirstACFail = TRUE;
	m_bFirstBattLow = TRUE;
	memset(&m_ShutdownAgent,0,sizeof(m_ShutdownAgent));
}

CUpsCheck::~CUpsCheck(void)
{
	if (m_hFileMap)
	{
		CloseHandle(m_hFileMap);
	}

	if (m_hMutex)
	{
		CloseHandle(m_hMutex);
	}

	if (m_hUpdateConfig)
	{
		CloseHandle(m_hUpdateConfig);
	}

	if (m_hUpdatePip)
	{
		CloseHandle(m_hUpdatePip);
	}
}
BOOL CUpsCheck::Initialize()
{
	m_bWrited = FALSE;
	m_iShutdownStep = 0;

	SECURITY_ATTRIBUTES sa;
	SECURITY_DESCRIPTOR sd;


	m_nWinVista = GetOSVersion_Vista();

	sa.nLength = sizeof(SECURITY_ATTRIBUTES);
	sa.bInheritHandle = TRUE;
	sa.lpSecurityDescriptor = &sd;

	if(!InitializeSecurityDescriptor(&sd,
		SECURITY_DESCRIPTOR_REVISION))
	{
		return false;   // Handle errors.
	}

	// Add a NULL DACL to the security descriptor.
	if(!SetSecurityDescriptorDacl(&sd, TRUE, (PACL)NULL, FALSE))
	{
		return false;   // Handle errors.
	}

	m_hUpdateConfig = CreateEvent(&sa, TRUE, FALSE, UPDATE_CONFIG_FILE);
	if ( m_hUpdateConfig == NULL )
	{
		return FALSE;
	}

	m_fileConfig.GetShutdownParameter(&m_SDParam);

	m_bSMUPSCmdInited = m_pReadMem.InitializeMem(1, sizeof(SHM_DATA), UPSDATAMAPPINGNAME, SYNCFILEMAPPING, TRUE);

	if( ! CreateMyNamedPipe())
		return FALSE;
	if( ! ConnectPipeClient())
		return FALSE;

	m_upsData = new CUPSDLL();

	if(m_upsData != NULL)
	{
		if(!m_upsData->Initialize())
		{
			m_upsData->Terminate();
			return FALSE;
		}

		if(!SetSecuAttr())
		{
			FreeRes();
		}

		if(m_hFileMap == NULL)
		{
			m_hFileMap = InitialShareMemory();
		}

		if(m_hMutex == NULL)
		{
			m_hMutex = InitialMutex();
		}
		
		return TRUE;
	}

	return FALSE;
}

BOOL CUpsCheck::SetSecuAttr()
{
	DWORD						dwRes;
	EXPLICIT_ACCESS				ea[2];
	SID_IDENTIFIER_AUTHORITY	SIDAuthWorld = SECURITY_WORLD_SID_AUTHORITY;
	SID_IDENTIFIER_AUTHORITY	SIDAuthNT = SECURITY_NT_AUTHORITY;
	BOOL						bRet = FALSE;

	// Create a well-known SID for the Everyone group.
	if(AllocateAndInitializeSid( &SIDAuthWorld, 1,
		SECURITY_WORLD_RID,
		0, 0, 0, 0, 0, 0, 0,
		&m_pEveryoneSID))
	{
		// Initialize an EXPLICIT_ACCESS structure for an ACE.
		// The ACE will allow Everyone read access to the key.
		ZeroMemory(&ea, 2 * sizeof(EXPLICIT_ACCESS));
		ea[0].grfAccessPermissions = STANDARD_RIGHTS_ALL | GENERIC_ALL | GENERIC_READ | GENERIC_WRITE;
		ea[0].grfAccessMode = SET_ACCESS;
		ea[0].grfInheritance= NO_INHERITANCE;
		ea[0].Trustee.TrusteeForm = TRUSTEE_IS_SID;
		ea[0].Trustee.TrusteeType = TRUSTEE_IS_WELL_KNOWN_GROUP;
		ea[0].Trustee.ptstrName  = (LPTSTR) m_pEveryoneSID;

		// Create a SID for the BUILTIN\Administrators group.
		if(AllocateAndInitializeSid( &SIDAuthNT, 2,
			SECURITY_BUILTIN_DOMAIN_RID,
			DOMAIN_ALIAS_RID_ADMINS,
			0, 0, 0, 0, 0, 0,
			&m_pAdminSID)) 
		{
			// Initialize an EXPLICIT_ACCESS structure for an ACE.
			// The ACE will allow the Administrators group full access to the key.
			ea[1].grfAccessPermissions = GENERIC_ALL | GENERIC_READ | GENERIC_WRITE | STANDARD_RIGHTS_ALL | SPECIFIC_RIGHTS_ALL;
			ea[1].grfAccessMode = SET_ACCESS;
			ea[1].grfInheritance= NO_INHERITANCE;
			ea[1].Trustee.TrusteeForm = TRUSTEE_IS_SID;
			ea[1].Trustee.TrusteeType = TRUSTEE_IS_GROUP;
			ea[1].Trustee.ptstrName  = (LPTSTR) m_pAdminSID;

			// Create a new ACL that contains the new ACEs.
			dwRes = SetEntriesInAcl(2, ea, NULL, &m_pACL);
			if (ERROR_SUCCESS == dwRes) 
			{
				// Initialize a security descriptor.  
				m_pSD = (PSECURITY_DESCRIPTOR) LocalAlloc(LPTR, SECURITY_DESCRIPTOR_MIN_LENGTH); 
				if (m_pSD) 
				{ 
					if (InitializeSecurityDescriptor(m_pSD, SECURITY_DESCRIPTOR_REVISION)) 
					{  
						// Add the ACL to the security descriptor. 
						if (SetSecurityDescriptorDacl(  m_pSD, 
							TRUE,     // bDaclPresent flag   
							m_pACL, 
							FALSE))   // not a default DACL 
						{
							bRet = TRUE;
						} 
					}
				} 
			} 
		}
	}

	// Initialize a security attributes structure.
	m_sa.nLength = sizeof (SECURITY_ATTRIBUTES);
	m_sa.lpSecurityDescriptor = m_pSD;
	m_sa.bInheritHandle = FALSE;

	return bRet;
}

void CUpsCheck::FreeRes()
{
	if (m_pEveryoneSID) 
	{
		FreeSid(m_pEveryoneSID);
		m_pEveryoneSID = NULL; 
	}

	if (m_pAdminSID) 
	{
		FreeSid(m_pAdminSID);
		m_pAdminSID = NULL;
	}

	if (m_pACL) 
	{
		LocalFree(m_pACL);
		m_pACL = NULL;
	}

	if (m_pSD) 
	{
		LocalFree(m_pSD);
		m_pSD = NULL;
	}
}

HANDLE CUpsCheck::InitialShareMemory ()
{
	DWORD	dwLen = sizeof(SHUTDOWN_AGENT);
	HANDLE	handle;


	handle = CreateFileMapping(
		NULL,
		&m_sa,
		PAGE_READWRITE,
		0, 
		dwLen,
		UPSSTATUSSHMNAME);
	

	if ( handle == NULL )
	{
		GetLastError();
		return NULL;
	}

	return handle;
}

HANDLE CUpsCheck::InitialMutex()
{
	HANDLE handle;

	handle = CreateMutex(&m_sa, FALSE, 
		UPSSTATUSSYNCFILEMAPPING);

	return handle;
}

BOOL CUpsCheck::ReadMemData()
{
	memset(&m_SHMData, 0, sizeof(m_SHMData));
	if(m_pReadMem.Lock(500))
	{
		if( !m_pReadMem.GetData(0, &m_SHMData, SHM_DATA_LENGTH) )
			return FALSE;

		m_pReadMem.Unlock();

		return TRUE;
	}
	else
		return FALSE;
}

BOOL CUpsCheck::ModifyUPSData(SHUTDOWN_AGENT &shutdownAgent)
{
	char	*pszBuf = NULL;
	int		i = 0;

	if(m_hFileMap == NULL)
	{
		m_hFileMap = InitialShareMemory();
	}
	if(m_hMutex == NULL)
	{
		m_hMutex = InitialMutex();
	}

	if( (m_hFileMap == NULL) || (m_hMutex == NULL))
	{
		return FALSE;
	}

	while(i++ < 25)
	{
		if(WaitForSingleObject(m_hMutex, 20) == WAIT_OBJECT_0 )
		{
			pszBuf = (char*) MapViewOfFileEx(m_hFileMap,
				FILE_MAP_WRITE,
				0,
				0,
				0,
				NULL);
			if(pszBuf == NULL)
			{
				ReleaseMutex(m_hMutex);
				return FALSE;
			}
			memcpy(pszBuf,(void *)&shutdownAgent, sizeof(SHUTDOWN_AGENT));
			UnmapViewOfFile(pszBuf);
			ReleaseMutex(m_hMutex);
			return TRUE;
		}
	}

	return FALSE;
}

BOOL CUpsCheck::CreateMyNamedPipe()
{
	m_hNamedPipe=CreateNamedPipe(_T(SZ_NAMED_PIPE),
		         PIPE_ACCESS_DUPLEX|FILE_FLAG_OVERLAPPED,
		         PIPE_TYPE_MESSAGE|PIPE_READMODE_MESSAGE,
		         2,256,256,10000,NULL);
	if (m_hNamedPipe==INVALID_HANDLE_VALUE)
	{
		DWORD dwErr=GetLastError();
		return FALSE;
	}
	else
		return TRUE;
}
BOOL CUpsCheck::ConnectPipeClient()
{
	m_overlapped.Offset = m_overlapped.OffsetHigh = 0;
	m_hConnectClient = CreateEvent(NULL,TRUE,FALSE,NULL);
	m_overlapped.hEvent = m_hConnectClient;

	if (m_hConnectClient == NULL)
	{
		return FALSE;
	}

	if (!ConnectNamedPipe(m_hNamedPipe,&m_overlapped))
		if (GetLastError()!=ERROR_IO_PENDING)
		{
			return FALSE;
		}
	if(WAIT_FAILED == WaitForSingleObject(m_hConnectClient, 20))
	{
		return FALSE;
	}
	return TRUE;
}

BOOL CUpsCheck::WritePipeCmd()
{
	DWORD dwWriteNum = 0;
	int nRet = 0;
	DWORD   length; 

	if (!m_bWrited)
	{
		memcpy(m_PipeData.szCmd, "Shutdown", sizeof(_T("Shutdown")));
		m_PipeData.nValue = m_SDParam.uShutdownNeedTime + m_SDParam.uCommandDelayTime;

		if((m_hUpdatePip = OpenEvent(EVENT_ALL_ACCESS, FALSE, READPIP_EVENT_NAME)) == NULL)
		{
			return FALSE;
		}
		 
		nRet = WriteFile(m_hNamedPipe, &m_PipeData, sizeof(PIPE_DATA), &dwWriteNum, &m_overlapped);
		if(!nRet && ::GetLastError()   ==   ERROR_IO_PENDING)  
		{
			nRet = GetOverlappedResult(m_hNamedPipe,   &m_overlapped,   &length,   TRUE);  
		}   

		

		if( ! nRet)
		{
			return FALSE;
		}
		m_bWrited = TRUE;

		SetEvent(m_hUpdatePip);
	}

	return TRUE;
}


int CUpsCheck::CheckACFail()
{
	time_t 	uPassedSeconds = 0;

	if((m_dwUPSStatus & UPS_STATUS_ACFAIL) || 
		(m_dwUPSStatus & UPS_STATUS_SD_OS_UPS) ||
		(m_dwUPSStatus & UPS_STATUS_SD_OS_RESTART_LATER) ||
		(m_dwUPSStatus & UPS_STATUS_ONLY_SD_OS))
	{
		if(!m_bACFail)
		{
			m_bACFail = TRUE;
			m_tACFailStart = time(NULL);
		}
		return 1;
	}
	else if(m_bACFail && !(m_dwUPSStatus & UPS_STATUS_SHUTDOWN))
	{
		if (m_iShutdownStep == 0)
		{
			SimpleSDLogEvent(EVENTLOG_INFORMATION_TYPE, UPS_EVENT_ACFAIL_RECOVER, ID_UPS_ACFAIL_RECOVER);
		}
		m_ShutdownAgent.iACFailStatus = ACFAILSTATUS_NONE;
		m_bFirstACFail = TRUE;
		m_bACFail = FALSE;
		m_tACFailStart = 0;
	}
	
	return 0;
} 

BOOL CUpsCheck::DoDelay()
{
	static int i = 0;
	time_t tNow;
	time_t uPassedSeconds;
	//orangexie add at 20101009 for battery low
	if (m_bBatteryLow)
	{
		tNow = time(NULL);
		uPassedSeconds = tNow - m_tACFailStart;
		WritePipeCmd();
		if (uPassedSeconds > TIME_RANGE)
		{
			return FALSE;
		}
		return TRUE;
	}

	if(m_bACFail)
	{
		//imachine added 2010.07.15
		if ( m_upsData->m_upsObject.m_nOmronStatus != 0 )
		{
			WritePipeCmd();
			return FALSE;
		}

		tNow = time(NULL);
		uPassedSeconds = tNow - m_tACFailStart;
		if (((time_t)m_SDParam.uACFailDelayTime - uPassedSeconds) < TIME_RANGE)
		{
			WritePipeCmd();
		}
		if(uPassedSeconds >= (time_t)m_SDParam.uACFailDelayTime)
		{	
			return FALSE;
		}	
		return TRUE;
	}
	else
	{
		//imachine added 2010.07.14
		SetEvent( m_upsData->m_hUPSCMDSNEvent );
		return FALSE;
	}
}

int CUpsCheck::CheckBattLow()
{
	if(m_dwUPSStatus & UPS_STATUS_BATTLOW)
	//we should shutdown system os and ups while battery low 
	//orangexie@20101009
	{
		if (!m_bBatteryLow)
		{
			m_bBatteryLow = TRUE;
			m_tACFailStart = time(NULL);
		}
		return 1;		
	}
	return 0;
}

int CUpsCheck::CheckEvent()
{
	int iRet = 0;

	m_ShutdownAgent.iACFailStatus = ACFAILSTATUS_NONE;

	if( ReadMemData() && m_SHMData.uDataAvailable == 1) // == 1 is  SHM_DATA_AVAILABLE
	{
		if (m_bFirstCommEst)
		{
			m_SDParam.uPortType = m_SHMData.uPortType;
			m_SDParam.uComPort = m_SHMData.uPortNum;
			m_fileConfig.SetShutdownParameter(&m_SDParam);

			//imachine added 2010.07.12
			SetEvent( m_upsData->m_hUpdateConnection );
			
			if(m_iShutdownStep == 0)
			{
				SimpleSDLogEvent(EVENTLOG_INFORMATION_TYPE, UPS_EVENT_COMM_EST, ID_UPS_COMM_EST);
			}
			m_bFirstCommEst = FALSE;
		}	

		m_dwUPSStatus = m_SHMData.dwUPSStatus;

		m_ShutdownAgent.iACFailStatus = UPSSTAUTS_COMEST;

		if( CheckACFail())
		{
			if (m_bFirstACFail)
			{
				if(m_iShutdownStep == 0)
				{
					if (m_dwUPSStatus & UPS_STATUS_ACFAIL)
						SimpleSDLogEvent(EVENTLOG_WARNING_TYPE, UPS_EVENT_ACFAIL, ID_UPS_ACFAIL);
					else if (m_dwUPSStatus & UPS_STATUS_SD_OS_UPS)
						SimpleSDLogEvent(EVENTLOG_WARNING_TYPE, UPS_EVENT_SD_OS_AND_UPS_BY_OMRON, ID_SD_OS_UPS_BY_OMRON);
					else if (m_dwUPSStatus & UPS_STATUS_SD_OS_RESTART_LATER)
						SimpleSDLogEvent(EVENTLOG_WARNING_TYPE, UPS_EVENT_SD_OS_RESTART_LATER_BY_OMRON, ID_SD_OS_RESTART_LATER_BY_OMRON);
					else if (m_dwUPSStatus & UPS_STATUS_ONLY_SD_OS)
						SimpleSDLogEvent(EVENTLOG_WARNING_TYPE, UPS_EVENT_ONLY_SD_OS_BY_OMRON, ID_ONLY_SD_OS_BY_OMRON);
				}
				m_bFirstACFail = FALSE;
			}

			if (m_dwUPSStatus & UPS_STATUS_ACFAIL)
			{
				m_ShutdownAgent.iACFailStatus = ACFAILSTATUS_ACFAIL;
			}
			else if (m_dwUPSStatus & UPS_STATUS_SD_OS_UPS)
			{
				m_ShutdownAgent.iACFailStatus = UPSSTATUS_SD_OS_UPS_BY_OMRON;
			}
			else if (m_dwUPSStatus & UPS_STATUS_SD_OS_RESTART_LATER)
			{
				m_ShutdownAgent.iACFailStatus = UPSSTATUS_SD_OS_RESTART_LATER_BY_OMRON;
			}
			else if (m_dwUPSStatus & UPS_STATUS_ONLY_SD_OS)
			{
				m_ShutdownAgent.iACFailStatus = UPSSTATUS_ONLY_SD_OS_BY_OMRON;
			}
			else
			{
				m_ShutdownAgent.iACFailStatus = ACFAILSTATUS_ACFAIL;
			}

			
			iRet = 1;
		}
		
		if( CheckBattLow())
		{
			if(m_bFirstBattLow)
			{
				if (m_iShutdownStep == 0)
				{
					SimpleSDLogEvent(EVENTLOG_WARNING_TYPE, UPS_EVENT_BATTERY_LOW, ID_UPS_BATTERY_LOW);
				}
				m_bFirstBattLow = FALSE;
			}
			
			m_ShutdownAgent.iACFailStatus = UPSSTATUS_BATTLOW;
			iRet = 1;
		}
	}
	else
	{
		if (!m_bFirstCommEst)
		{
			if (m_iShutdownStep == 0)
			{
				SimpleSDLogEvent(EVENTLOG_INFORMATION_TYPE, UPS_EVENT_COMM_LOST, ID_UPS_COMM_LOST);
			}
			m_bFirstCommEst = TRUE;
		}	
		m_ShutdownAgent.iACFailStatus = UPSSTATUS_COMLST;
	}
	goble_variable_UPS_status = m_ShutdownAgent.iACFailStatus; //Colin added for get ups status

	Sleep(5000);

	ModifyUPSData(m_ShutdownAgent);
	
	return iRet;
}

int CUpsCheck::WriteEventLog(char *pszBufMes)
{
	char 	szBuf[MAX_LINELENGTH];
	DWORD 	dwRowCount = 0;
	struct 	tm *ptm = NULL;
	time_t	tNow = 0;
	FILE	* fp = NULL;
	FILE	* fpTmp = NULL;
	char	FilePath[MAX_PATH];
	char	szLogBuf[MAX_LINELENGTH];

	memset(FilePath,0,MAX_PATH);

	tNow = time(NULL);
	ptm = localtime(&tNow);
	if(!ptm)
	{
		return -1; 
	}

	memset(szBuf, 0, sizeof(szBuf));

	sprintf_s(szBuf,MAX_LINELENGTH,
		"%04d/%02d/%02d %02d:%02d:%02d %s\n",
		ptm->tm_year+1900,
		ptm->tm_mon+1,
		ptm->tm_mday,
		ptm->tm_hour,
		ptm->tm_min,
		ptm->tm_sec,
		pszBufMes);

	m_fileConfig.GetFilePath(FilePath, EVENTLOG_FILE_NAME);

	fopen_s(&fp,FilePath,"a+t");
	if (fp == NULL)
	{
		return -1;
	}

	memset(szLogBuf, 0, sizeof(szLogBuf));	

	while(fgets(szLogBuf, MAX_LINELENGTH, fp))
	{
		dwRowCount++;
	}

	if(dwRowCount < MAX_EVENTLOG_COUNT)
	{
		fwrite(szBuf, strlen(szBuf), 1, fp);
		fflush(fp);
		fclose(fp);
	}
	else
	{
		fseek(fp,0,SEEK_SET);

		fopen_s(&fpTmp, TEMP_FILE, "w");
		if(fpTmp == NULL)
			return -1;
		while(dwRowCount >= MAX_EVENTLOG_COUNT)
		{
			fgets(szLogBuf,MAX_LINELENGTH, fp);
			dwRowCount--;
		}
		while(fgets(szLogBuf, MAX_LINELENGTH, fp))
		{
			fwrite(szLogBuf,strlen(szLogBuf),1,fpTmp);
		}
		fwrite(szBuf, strlen(szBuf), 1, fpTmp);

		fclose(fp);
		fflush(fpTmp);
		fclose(fpTmp);
		if(remove(FilePath) != 0)
		{
			return 0;
		}
		if(rename(TEMP_FILE,FilePath) != 0)
		{
			return 0;
		}
	}

	return 1;
}
void Substitute(char *pInput, char *pOutput, char *pSrc, char *pDst)
{
	char *pi = NULL;
	char *po = NULL;
	char *p = NULL;
	size_t nSrcLen = 0;
	size_t nDstLen = 0;
	size_t nLen = 0;
	pi = pInput;
	po = pOutput;
	nSrcLen = strlen(pSrc);
	nDstLen = strlen(pDst); 
	p = strstr(pi, pSrc);
	if(p)
	{
		while(p)
		{
			nLen = (int)(p - pi);
			memcpy(po, pi, nLen);
			memcpy(po + nLen, pDst, nDstLen);
			pi = p + nSrcLen;
			po = po + nLen + nDstLen;
			p = strstr(pi, pSrc);
		}
		strcpy(po, pi);
	}
	else
	{
		strcpy(po, pi);
	}
}
void CUpsCheck::SimpleSDLogEvent(WORD wType, DWORD dwID,UINT strResID)
{
	HANDLE hEventLog = NULL;
	TCHAR *pMsgBuf[1];
	DWORD dwEventID;
	size_t		i = 0;
	size_t		nSize = MAX_PATH;
	char	szFileName[MAX_PATH] = {0};
	FILE * fp = NULL;
	char szTemp[512];

	char szMesStr[MAX_LINELENGTH] = {0};
	char szString[512];

	memset(szMesStr,0,MAX_LINELENGTH);
	
	if (!GetEventParamString(strResID, &dwEventID, szMesStr))
	{
		return;
	}

	if (ID_SD_OS_RESTART_LATER_BY_OMRON == strResID)
	{
		memset(szTemp, 0x00, sizeof(szTemp));
		
		if(GetModuleFileName(NULL, szFileName, (DWORD)nSize))
		{
			for(i=strlen(szFileName); i>=0; i--)
			{
				if(szFileName[i]== '\\')
					break;
			}
			
			szFileName[i+1] = 0;
			
			strcat(szFileName, "omronctl.ini");
		}
		fp = fopen(szFileName, "r");
		if (NULL != fp)
		{
			memset(szTemp, 0x00, sizeof(szTemp));
			fread(szTemp, sizeof(szTemp), 1, fp);
			fclose(fp);
			fp = NULL;
		}
		memset(szString, 0x00, sizeof(szString));
		Substitute(szMesStr, szString, "%1", szTemp);
		memset(szMesStr, 0x00, sizeof(szMesStr));
		strcpy(szMesStr, szString);
	}

	hEventLog = RegisterEventSource(NULL, _T("Simple Shutdown"));


	pMsgBuf[0] = szMesStr;

	if(NULL != hEventLog)
	{
		BOOL bReport = ReportEvent(hEventLog,
			wType,
			0,
			dwID,
			NULL,
			1,
			0,
			(LPCSTR * )pMsgBuf,
			NULL);
	}
	DeregisterEventSource(hEventLog);

	WriteEventLog(szMesStr);
}

void CUpsCheck::UpdateConfigFile()
{	
	if (WaitForSingleObject(m_hUpdateConfig, 100) == WAIT_OBJECT_0)
	{
		m_fileConfig.GetShutdownParameter(&m_SDParam);
		ResetEvent(m_hUpdateConfig);
	}
}


BOOL CUpsCheck::GetTokenByName(HANDLE &hToken)
{
	HANDLE         hProcessSnap = NULL; 
	BOOL           bRet			= FALSE; 
	PROCESSENTRY32 pe32			= {0}; 

	hProcessSnap = CreateToolhelp32Snapshot(TH32CS_SNAPPROCESS, 0); 
	if (hProcessSnap == INVALID_HANDLE_VALUE) 
	{
		return FALSE;
	}

	pe32.dwSize = sizeof(PROCESSENTRY32); 

	if (Process32First(hProcessSnap, &pe32)) 
	{  
		do 
		{
			//imachine modified 2010.07.09
			if ( (!strcmp(_strupr(pe32.szExeFile), _strupr("EXPLORER.EXE"))) || ((!strcmp(_strupr(pe32.szExeFile), _strupr("CMD.EXE")))) )
			{
				HANDLE hProcess = OpenProcess(PROCESS_QUERY_INFORMATION, FALSE, pe32.th32ProcessID);
				if(hProcess)
				{
					bRet = OpenProcessToken(hProcess, TOKEN_ALL_ACCESS, &hToken);
					CloseHandle (hProcess);
				}
				CloseHandle (hProcessSnap);
				return bRet;
			}
		} 
		while (Process32Next(hProcessSnap, &pe32)); 
	} 
	else 
	{
		bRet = FALSE;
	}

	CloseHandle (hProcessSnap); 
	return bRet;
}

BOOL CUpsCheck::DoShutdownLogic()
{
	LPVOID					pEnv = NULL;
	time_t tNow = time(NULL);
	if(m_iShutdownStep == 0)
	{
		m_fileConfig.GetShutdownParameter(&m_SDParam);

		SimpleSDLogEvent(EVENTLOG_INFORMATION_TYPE, UPS_EVENT_EXCMD, ID_RUN_EXTERNCMD);

		if(m_nWinVista >= 0)	//1->vista ,0->Windows Server 2008
		{
			BOOL				bRet = FALSE;
			HANDLE				hToken = NULL;
			STARTUPINFO			startupInfo;
			PROCESS_INFORMATION proInfo;
			
			GetTokenByName(hToken);	

			if(hToken)
			{
				/*char szTmpExternalCmd[MAX_PATH] = {0};
				if(m_SDParam.szExternCmd[0] != '\"')
					sprintf_s(szTmpExternalCmd, MAX_PATH, "\"%s\"", m_SDParam.szExternCmd);
				else
					sprintf_s(szTmpExternalCmd, MAX_PATH, "%s", m_SDParam.szExternCmd);*/

				char szTmpExternalCmd[MAX_PATH] = {0};
				sprintf_s(szTmpExternalCmd, MAX_PATH, "%s", m_SDParam.szExternCmd);

				ZeroMemory(&startupInfo,sizeof(STARTUPINFO));
				startupInfo.cb = sizeof(STARTUPINFO);
				startupInfo.lpDesktop = TEXT("winsta0\\default");
				startupInfo.lpTitle=NULL;          
				startupInfo.dwFlags=0;   

				CreateEnvironmentBlock(&pEnv,hToken,FALSE);
				bRet=CreateProcessAsUser(hToken, szTmpExternalCmd, NULL, NULL, NULL, FALSE,
					CREATE_NEW_PROCESS_GROUP | CREATE_UNICODE_ENVIRONMENT ,
					pEnv, NULL, &startupInfo, &proInfo);
				DestroyEnvironmentBlock(pEnv);

				if( bRet == FALSE )
				{
					char str[64] = {0};
					sprintf_s(str,64,"GetLastError=%d",GetLastError()); 
					ExecuteExternCmd();
				}

				CloseHandle(hToken);
			}
			
		}
		else
		{
			ExecuteExternCmd();
		}
		m_tStartShutdownTime = time(NULL);

		m_iShutdownStep = 1;
	}

	if (m_iShutdownStep == 1)
	{
		if(((time_t)(m_SDParam.uCommandDelayTime+m_tStartShutdownTime)) > tNow)
		{
			return FALSE;
		}
	}
	
	m_iShutdownStep = 0;
	m_tStartShutdownTime = 0;
	
	return TRUE;
}

BOOL CUpsCheck::ExecuteExternCmd()
{
	TCHAR szTmpExternalCmd[MAX_PATH] = {0};
	STARTUPINFO startupInfo;
	PROCESS_INFORMATION proInfo;

	if(!strlen(m_SDParam.szExternCmd))
	{
		return FALSE;
	}

	memset(&startupInfo, 0, sizeof(startupInfo));
	memset(&proInfo, 0, sizeof(proInfo));

	if(m_SDParam.szExternCmd[0] != '\"')
		sprintf_s(szTmpExternalCmd, MAX_PATH, "\"%s\"", m_SDParam.szExternCmd);
	else
		sprintf_s(szTmpExternalCmd, MAX_PATH, "%s", m_SDParam.szExternCmd);

	startupInfo.cb = sizeof(STARTUPINFO);
	GetStartupInfo(&startupInfo);
	return CreateProcess(NULL, szTmpExternalCmd, NULL, NULL, FALSE,
		CREATE_NEW_PROCESS_GROUP | CREATE_NEW_CONSOLE,
		NULL, NULL, &startupInfo, &proInfo);
}




int LOGONEMSGEX(char *pszBuff)
{
#if 0
	char szTemp[512] = {0};
	FILE * fp = NULL;
	time_t tNow = time(NULL);
	tm * stTime = localtime(&tNow);
	size_t i = 0;
	char szFileName[MAX_PATH] = {0};
	int nSize = MAX_PATH;

	int year = stTime->tm_year + 1900;
	int month = stTime->tm_mon + 1;
	int day = stTime->tm_mday;
	int nHour = stTime->tm_hour;
	int nMin = stTime->tm_min;
	int nSec = stTime->tm_sec;

	sprintf(szTemp, "%04d/%02d/%02d %02d:%02d:%02d   %s\n", 
		year,
		month,
		day,
		nHour,
		nMin,
		nSec,
		pszBuff);



	if(GetModuleFileName(NULL, szFileName, nSize))
	{
		for(i = strlen(szFileName); i >= 0; i --)
		{
			if(szFileName[i] == '\\')
				break;
		}

		szFileName[i+1] = 0;
	}
	strcat(szFileName, "SimpleSD.log");


	fp = fopen(szFileName, "at +");
	if (NULL != fp)
	{
		fwrite(szTemp, 1, strlen(szTemp), fp);
		fclose(fp);
	}
#endif
	return 0;
}
